<?php

namespace Drupal\reverse_proxy_header\EventSubscriber;

use Drupal\Core\Site\Settings;
use GuzzleHttp\ClientInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Restores the true client IP address.
 */
class ReverseProxyHeaderClientIpRestore implements EventSubscriberInterface {

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The settings object.
   *
   * @var \Drupal\Core\Site\Settings
   */
  protected $settings;

  /**
   * Constructs a ReverseProxyHeaderClientIpRestore.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   A Guzzle client object.
   * @param \Drupal\Core\Site\Settings $settings
   *   The settings object.
   */
  public function __construct(ClientInterface $http_client, Settings $settings) {
    $this->httpClient = $http_client;
    $this->settings = $settings;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['onRequest', 20];
    return $events;
  }

  /**
   * Restores the origination client IP from the custom HTTP header.
   */
  public function onRequest(RequestEvent $event) {

    // Check the available settings.
    $reverse_proxy_header_name = $this->settings->get('reverse_proxy_header');
    if (!$reverse_proxy_header_name) {
      return;
    }

    // Check the header value.
    $current_request = $event->getRequest();
    $connecting_ip = $current_request->server->get($reverse_proxy_header_name);
    if (empty($connecting_ip)) {
      return;
    }

    // As the changed remote address will make it impossible to determine
    // a trusted proxy, we need to make sure we set the right protocol as well.
    // @see \Symfony\Component\HttpFoundation\Request::isSecure()
    $event->getRequest()->server->set('HTTPS', $event->getRequest()->isSecure() ? 'on' : 'off');
    $event->getRequest()->server->set('REMOTE_ADDR', $connecting_ip);
    $event->getRequest()->overrideGlobals();
  }

}
